from SocketServer import UnixStreamServer, TCPServer, StreamRequestHandler
from os import chown, chmod
import traceback, socket
import sys


class StopServer(Exception):
    def __init__(self, response):
        self.response = response


class ValidationFailed(Exception):
    def __init__(self, response):
        self.response = response


class PacketHandler(StreamRequestHandler):
    def handle(self):
        request = self.rfile.read()
        self.processPacket(request)

    def write_response(self, response):
        try:
            self.wfile.write(response)
            self.wfile.flush()
        except: # should be logged
            pass


class ValidatedPacketHandler(PacketHandler):
    def processPacket(self, request):
        try:
            try:
                response = self.handleRequest(request)
            except StopServer, e:
                response = e.response
                raise
            except:
                response = self.processError()
        finally:
            self.handleResponse(response)

    def handleRequest(self, request):
        try:
            self.validateRequest(request)
        except ValidationFailed, e:
            return e.response
        request = self.deserializeRequest(request)
        return self.do(request)

    def handleResponse(self, response):
        response = self.serializeResponse(response)
        self.validateResponse(response)
        self.write_response(response)


class PacketServer(UnixStreamServer):
    def __init__(self, socket, handler, socket_owner, socket_filemode):
        self.socket_owner = socket_owner
        self.socket_filemode = socket_filemode
        UnixStreamServer.__init__(self, socket, handler)

    def server_activate(self):
        chown(self.server_address, -1, self.socket_owner)
        chmod(self.server_address, self.socket_filemode)
        TCPServer.server_activate(self)

    def handle_request(self):
        try:
            request, client_address = self.get_request()
        except socket.error:
            return

        try:
            self.process_request(request, client_address)
        finally:
            self.close_request(request)
