import libxml2, libxml2mod, logging


_logger = logging.getLogger("api.util.validator")


class ValidatorErrorHandler:
    def __init__(self):
        self.errors = []
    def handler(self, msg, data):
        self.errors.append(msg)
    def __unicode__(self):
        return u"\n".join(self.errors)

#
# Hack. The python libxml2 wrappers are too old on supported OSes and
# do not provide such a function.
#
# FIXME: built newer wrappers. check whether this is still needed
#
def register_error_handler(ctxt_valid, err_func, warn_func, arg = None):
    libxml2mod.xmlSchemaSetValidErrors(ctxt_valid._o, err_func, warn_func, arg)


class Validator:
    def __init__(self, schemafile):
        self.ctxt_parser = libxml2.schemaNewParserCtxt(schemafile)
        self.ctxt_schema = self.ctxt_parser.schemaParse()

    def __del__(self):
        del self.ctxt_schema
        del self.ctxt_parser

        libxml2.schemaCleanupTypes()
        libxml2.cleanupParser()

    def do(self, xmlstr):
        ctxt_valid = self.ctxt_schema.schemaNewValidCtxt()
        erh = ValidatorErrorHandler()

        register_error_handler(ctxt_valid, erh.handler, erh.handler)

        doc = libxml2.parseDoc(xmlstr)
        ret = doc.schemaValidateDoc(ctxt_valid)
        doc.freeDoc()
        del ctxt_valid
        del doc

        if ret != 0:
            _logger.error("Validation failed:\n" + unicode(erh))
        else:
            _logger.info("Packet succesfully validated.")

        return (ret, unicode(erh))
